
#include <stdbool.h>


static double dispKp;				// * we'll hold on to the tuning parameters in user-entered 
static double dispKi;				//   format for display purposes
static double dispKd;				//
    
static double kp;                  // * (P)roportional Tuning Parameter
static double ki;                  // * (I)ntegral Tuning Parameter
static double kd;                  // * (D)erivative Tuning Parameter

static int controllerDirection;
static int pOn;

static double *myInput;              // * Pointers to the Input, Output, and Setpoint variables
static double *myOutput;             //   This creates a hard link between the variables and the 
static double *mySetpoint;           //   PID, freeing the user from having to constantly tell us
                                  //   what these values are.  with pointers we'll just know.

static unsigned long lastTime;
static double outputSum, lastInput;

static unsigned long SampleTime;
static double outMin, outMax;
static bool inAuto, pOnE;



/* Compute() **********************************************************************
 *     This, as they say, is where the magic happens.  this function should be called
 *   every time "void loop()" executes.  the function will decide for itself whether a new
 *   pid Output needs to be computed.  returns true when the output is computed,
 *   false when nothing has been done.
 **********************************************************************************/
bool Compute()
{
    if(!inAuto)
        return false;

    unsigned long now = _getms();
    unsigned long timeChange = (now - lastTime);

    if(timeChange >= SampleTime)
    {
        /*Compute all the working error variables*/
        double input = *myInput;
        double error = *mySetpoint - input;
        double dInput = (input - lastInput);

        outputSum += (ki * error);

        /*Add Proportional on Measurement, if P_ON_M is specified*/
        if(!pOnE)
            outputSum-= kp * dInput;

        if(outputSum > outMax)
            outputSum = outMax;
        else if(outputSum < outMin)
            outputSum = outMin;

        /*Add Proportional on Error, if P_ON_E is specified*/
        double output;
        if(pOnE)
            output = kp * error;
        else
            output = 0;

        /*Compute Rest of PID Output*/
        output += outputSum - kd * dInput;

        if(output > outMax)
            output = outMax;
        else if(output < outMin)
            output = outMin;
        *myOutput = output;

        /*Remember some variables for next time*/
        lastInput = input;
        lastTime = now;
        return true;
    }
   else return false;
}



void main(void)
{
    Compute();
}
